@@ -22,6 +22,10 @@ class datacube(object):
             square field of view with sides of `FOV` [arcsec].
         velocity_range (Optional[list]): A tuple or list of the minimum and
             maximum velocities in [m/s] to clip the data cube to.
+        fill (Optional[float]): Replace all ``NaN`` values with this value.
+        force_center (Optional[bool]): If ``True`` define the spatial axes such
+            that they describe offset from the array center in [arcsec]. This
+            is useful if the FITS header does not contain axis information.
     """
 
     flared_niter = 5
@@ -32,12 +36,14 @@ class datacube(object):
     msun = 1.98847e30
     fwhm = 2. * np.sqrt(2 * np.log(2))
 
-    def __init__(self, path, FOV=None, velocity_range=None, fill=None):
+    def __init__(self, path, FOV=None, velocity_range=None, fill=None,
+                 force_center=False):
 
         # Read in the data
 
         self.path = path
-        self._read_FITS(path=self.path, fill=fill)
+        self._read_FITS(path=self.path, fill=fill,
+                        force_center=force_center)
 
         # Clip down the cube.
 
@@ -49,9 +55,9 @@ class datacube(object):
     # -- PIXEL DEPROJECTION -- #
 
     def disk_coords(self, x0=0.0, y0=0.0, inc=0.0, PA=0.0, z0=None, psi=None,
-                    r_cavity=0.0, r_taper=None, q_taper=None, w_i=0.0, w_r=1.0,
-                    w_t=0.0, z_func=None, outframe='cylindrical',
-                    shadowed=False, **_):
+                    r_cavity=0.0, r_taper=None, q_taper=1.0, z_func=None,
+                    outframe='cylindrical', shadowed=False, force_side=None,
+                    **_):
         r"""
         Get the disk coordinates given certain geometrical parameters and an
         emission surface. The emission surface is most simply described as a
@@ -157,6 +163,12 @@ class datacube(object):
                 coordinates. Either ``'cartesian'`` or ``'cylindrical'``.
             shadowed (Optional[bool]): Whether to use the slower, but more
                 robust method for deprojecting pixel values.
+            force_side (Optional[str]): Force the emission surface to be a
+                particular side, i.e., ``force_side='front'`` will force the
+                ``z_func`` argument to be positive, while ``force_side='back'``
+                will force ``z_func`` to be negative. Setting
+                ``force_side=None`` will allow any limit for the emission
+                height.
 
         Returns:
             array, array, array: Three coordinate arrays with ``(r, phi, z)``,
@@ -175,61 +187,23 @@ class datacube(object):
 
         inc = inc if inc < 90.0 else inc - 180.0
 
-        # Check that the necessary pairs are provided.
+        # Cycle through the different options for pixel deprojection.
 
-        msg = "Must specify either both or neither of `{}` and `{}`."
-        if (r_taper is not None) != (q_taper is not None):
-            raise ValueError(msg.format('r_taper', 'q_taper'))
-
-        msg = "Must specify all of the warp coorinates: (w_r, w_i, w_t)."
-        if (w_i is not None) != (w_t is not None) != (w_r is not None):
-            raise ValueError(msg)
-
-        flared = r_taper is not None or w_i is not None or r_cavity is not None
-
-        # Select the quickest pixel deprojection method.
-        # First round is analytical forms where `z_func` is not specified.
-
-        if z_func is None:
-            if z0 is None or z0 == 0.0:
-                r, t = self._get_midplane_polar_coords(x0, y0, inc, PA)
-                z = np.zeros(r.shape)
-            elif z0 > 0.0 and psi == 1.0 and not flared:
-                r, t, z = self._get_conical_polar_coords(x0, y0, inc, PA, z0)
-
-        # Here `z_func` can still be not provided, but must be defined based
-        # on the parameters provided (i.e., psi != 1.0).
-
-        r_cavity = 0.0 if r_cavity is None else r_cavity
-        r_taper = np.inf if r_taper is None else r_taper
-        q_taper = 1.0 if q_taper is None else q_taper
-        w_i = 0.0 if w_i is None else w_i
-        w_r = 1.0 if w_r is None else w_r
-        w_t = 0.0 if w_t is None else w_t
-
-        # Define the emission surface and warp functions.
-
-        if z_func is None:
-            def z_func(r_in):
-                r = np.clip(r_in - r_cavity, a_min=0.0, a_max=None)
-                z = r**psi * np.exp(-np.power(r / r_taper, q_taper))
-                return np.clip(z0 * z, a_min=0.0, a_max=None)
-
-        def w_func(r_in, t):
-            r = np.clip(r_in - r_cavity, a_min=0.0, a_max=None)
-            warp = np.radians(w_i) * np.exp(-0.5 * (r / w_r)**2)
-            return r * np.tan(warp * np.sin(t - np.radians(w_t)))
-
-        # Select the deprojection method. `shadowed` uses the slower (but more
-        # robust) forward modelling approach, rather than an iterative method.
-
-        if shadowed:
-            coords = self._get_shadowed_coords(x0, y0, inc, PA,
-                                               z_func, w_func)
+        if z0 is None:
+            r, t = self._get_midplane_polar_coords(x0, y0, inc, PA)
+            z = np.zeros(r.shape)
+        elif psi is None:
+            r, t, z = self._get_conical_polar_coords(x0, y0, inc, PA, z0)
         else:
-            coords = self._get_flared_coords(x0, y0, inc, PA,
-                                             z_func, w_func)
-        r, t, z = coords
+            if z_func is None:
+                r_taper = np.inf if r_taper is None else r_taper
+                def z_func(r_in):
+                    r = np.clip(r_in - r_cavity, a_min=0.0, a_max=None)
+                    return z0 * r**psi * np.exp(-np.power(r/r_taper, q_taper))
+            if shadowed:
+                r, t, z = self._get_shadowed_coords(x0, y0, inc, PA, z_func)
+            else:
+                r, t, z = self._get_flared_coords(x0, y0, inc, PA, z_func)
 
         # Return the values.
 
@@ -334,7 +308,7 @@ class datacube(object):
         x_d, y_d, z_d = self._get_conical_cart_coords(x0, y0, inc, PA, z0)
         return np.hypot(y_d, x_d), np.arctan2(y_d, x_d), z_d
 
-    def _get_flared_coords(self, x0, y0, inc, PA, z_func, w_func):
+    def _get_flared_coords(self, x0, y0, inc, PA, z_func, w_func=None):
         """Return cyclindrical coords of surface in [arcsec, rad, arcsec]."""
         x_mid, y_mid = self._get_midplane_cart_coords(x0, y0, inc, PA)
         r_tmp, t_tmp = np.hypot(x_mid, y_mid), np.arctan2(y_mid, x_mid)
@@ -344,16 +318,17 @@ class datacube(object):
             t_tmp = np.arctan2(y_tmp, x_mid)
         return r_tmp, t_tmp, z_func(r_tmp)
 
-    def _get_shadowed_coords(self, x0, y0, inc, PA, z_func, w_func):
+    def _get_shadowed_coords(self, x0, y0, inc, PA, z_func, w_func=None):
         """
         Return cyclindrical coords of surface in [arcsec, rad, arcsec].
         """
 
         # Make the disk-frame coordinates.
 
-        diskframe_coords = self._get_diskframe_coords()
-        xdisk, ydisk, rdisk, tdisk = diskframe_coords
-        zdisk = z_func(rdisk) + w_func(rdisk, tdisk)
+        xdisk, ydisk, rdisk, tdisk = self._get_diskframe_coords()
+        zdisk = z_func(rdisk)
+        if w_func is not None:
+            zdisk += w_func(rdisk, tdisk)
 
         # Incline the disk.
 
@@ -402,8 +377,8 @@ class datacube(object):
 
     def get_mask(self, r_min=None, r_max=None, exclude_r=False, phi_min=None,
                  phi_max=None, exclude_phi=False, abs_phi=False, x0=0.0,
-                 y0=0.0, inc=0.0, PA=0.0, z0=None, psi=None, r_cavity=None,
-                 r_taper=None, q_taper=None, w_i=None, w_r=None, w_t=None,
+                 y0=0.0, inc=0.0, PA=0.0, z0=0.0, psi=1.0, r_cavity=0.0,
+                 r_taper=np.inf, q_taper=1.0, w_i=None, w_r=None, w_t=None,
                  z_func=None, shadowed=False, mask_frame='disk',
                  user_mask=None):
         """
@@ -499,7 +474,7 @@ class datacube(object):
 
     # -- DATA I/O -- #
 
-    def _read_FITS(self, path, fill=None):
+    def _read_FITS(self, path, fill=None, force_center=False):
         """Reads the data from the FITS file."""
 
         # File names.
@@ -507,19 +482,27 @@ class datacube(object):
         self.path = os.path.expanduser(path)
         self.fname = self.path.split('/')[-1]
 
-        # FITS data.
+        # Read in the data and, if necessary, fill the NaNs with default
+        # values. Note that in the case of multiple data fields, we need to
+        # think of something different.
 
         self.header = fits.getheader(path)
         self.data = np.squeeze(fits.getdata(self.path))
         if fill is not None:
             self.data = np.where(np.isfinite(self.data), self.data, fill)
 
-        # Position axes.
+        # Position axes. Two options here, either try to build the axis based
+        # on the information in the header, or if force_center=True then return
+        # an axis where the offset is relative to the image center
 
-        self.xaxis = self._readpositionaxis(a=1)
-        self.yaxis = self._readpositionaxis(a=2)
-        self.xaxis -= 0.5*self.dpix
-        self.yaxis -= 0.5*self.dpix
+        if force_center:
+            self.xaxis = self._forcepositionaxis(a=1)
+            self.yaxis = self._forcepositionaxis(a=2)
+        else:
+            self.xaxis = self._readpositionaxis(a=1)
+            self.yaxis = self._readpositionaxis(a=2)
+            self.xaxis -= 0.5*self.dpix
+            self.yaxis -= 0.5*self.dpix
 
         # Spectral axis.
 
@@ -699,7 +682,16 @@ class datacube(object):
                 a_del = 1.0 * self._user_pixel_scale
             a_pix = a_len / 2.0 + 0.5
         axis = (np.arange(a_len) - a_pix + 1.0) * a_del
-        return 3600 * axis
+        return 3600.0 * axis
+
+    def _forcepositionaxis(self, a=1):
+        """Returns the axis in [arcsec] assuming image is centered."""
+        if a not in [1, 2]:
+            raise ValueError("'a' must be in [1, 2].")
+        a_len = self.data.shape[-1] if a == 1 else self.data.shape[-2]
+        axis = np.arange(a_len).astype('float') - a_len / 2.0
+        axis += 0.5 * (abs(axis[0]) - abs(axis[-1]))
+        return axis
 
     def _readrestfreq(self):
         """Read the rest frequency."""
@@ -733,6 +725,13 @@ class datacube(object):
             return self._readspectralaxis(a)
         return self._readrestfreq() * (1.0 - self._readvelocityaxis() / sc.c)
 
+    def force_relative_offset_axes(self):
+        """Force the use of relative offset axes."""
+        dx = (self.xaxis.max() - self.xaxis.min()) / 2.0
+        self.xaxis = np.linspace(dx, -dx, self.xaxis.size)
+        dy = (self.yaxis.max() - self.yaxis.min()) / 2.0
+        self.yaxis = np.linspace(-dy, dy, self.yaxis.size)
+
     # -- UNIT CONVERSIONS -- #
 
     def jybeam_to_Tb_RJ(self, data=None, nu=None):
