@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import time
+import yaml
 import zeus
 import emcee
 import numpy as np
@@ -34,21 +35,48 @@ class rotationmap(datacube):
             If you use ``downsample='beam'`` it will sample roughly
             spatially independent pixels using the beam major axis as the
             spacing.
+        fill (Optional[float]): Replace all ``NaN`` values with this value.
+        force_center (Optional[bool]): If ``True`` define the spatial axes such
+            that they describe offset from the array center in [arcsec]. This
+            is useful if the FITS header does not contain axis information.
     """
 
     priors = {}
+    SHO_priors = {}
 
-    def __init__(self, path, FOV=None, uncertainty=None, downsample=None):
-        datacube.__init__(self, path=path, FOV=FOV, fill=None)
-        self.data *= 1e3 if self.header['bunit'].lower() == 'km/s' else 1.0
+    def __init__(self, path, FOV=None, uncertainty=None, downsample=None,
+                 fill=None, force_center=False):
+        datacube.__init__(self, path=path, FOV=FOV, fill=fill,
+                          force_center=force_center)
+
+        # Check to see what unit the velocities are in.
+        # TODO: Sometimes the bunit is not m/s or km/s so we might want to
+        # include a check.
+
+        if self.header['bunit'].lower() not in ['m/s', 'km/s']:
+            msg = "What is the velocity unit? Either `m/s` or `km/s`."
+            self.velocity_unit = input(msg)
+        else:
+            self.velocity_unit = self.header['bunit'].lower()
+
+        self.data *= 1e3 if self.velocity_unit == 'km/s' else 1.0
         self.mask = np.isfinite(self.data)
         self._readuncertainty(uncertainty=uncertainty, FOV=FOV)
+
         if downsample is not None:
             self.downsample_cube(downsample)
-        self.vlsr = np.nanmedian(self.data)
-        self.vlsr_kms = self.vlsr / 1e3
+
+        self.default_parameters = self._load_default_parameters()
         self._set_default_priors()
 
+    @property
+    def vlsr(self):
+        return np.nanmedian(self.data)
+
+    @property
+    def vlsr_kms(self):
+        return self.vlsr / 1e3
+
     # -- FITTING FUNCTIONS -- #
 
     def fit_map(self, p0, params, r_min=None, r_max=None, optimize=True,
@@ -259,13 +287,13 @@ class rotationmap(datacube):
         return to_return if len(to_return) > 1 else to_return[0]
 
     def fit_annuli(self, rpnts=None, rbins=None, x0=0.0, y0=0.0, inc=0.0,
-                   PA=0.0, z0=None, psi=None, r_cavity=None, r_taper=None,
+                   PA=0.0, z0=0.0, psi=1.0, r_cavity=None, r_taper=None,
                    q_taper=None, w_i=None, w_r=None, w_t=None, z_func=None,
                    shadowed=False, phi_min=None, phi_max=None,
                    exclude_phi=False, abs_phi=False, mask_frame='disk',
                    user_mask=None, fit_vrot=True, fit_vrad=True,
                    fix_vlsr=False, plots=None, returns=None,
-                   optimize_kwargs=None):
+                   optimize_kwargs=None, MCMC=False):
         r"""
         Splits the map into concentric annuli based on the geometrical
         parameters, then fits each annnulus with a simple harmonic oscillator
@@ -391,6 +419,7 @@ class rotationmap(datacube):
                                            fit_vrot=fit_vrot,
                                            fit_vrad=fit_vrad,
                                            fix_vlsr=fix_vlsr,
+                                           MCMC=MCMC,
                                            optimize_kwargs=optimize_kwargs)
             except ValueError:
                 popt = np.ones(nparams) * np.nan
@@ -487,11 +516,10 @@ class rotationmap(datacube):
         return vrot * np.cos(pvals) + vrad * np.sin(pvals) + vlsr
 
     def _fit_SHO(self, x, y, dy, fit_vrot=True, fit_vrad=True, fix_vlsr=False,
-                 optimize_kwargs=None):
+                 optimize_kwargs=None, MCMC=False):
         """Fit the points with a simple harmonic oscillator."""
 
         from .helper_functions import SHO_double
-        from scipy.optimize import curve_fit
 
         # Guess the starting parameters.
 
@@ -506,48 +534,59 @@ class rotationmap(datacube):
                 def func(x, *params):
                     return SHO_double(x, *params)
                 p0 = [vrot, vrad, vlsr]
-                nparam = 3
+                priors = ['vrot', 'vrad', 'vlsr']
             elif fit_vrot and (not fit_vrad):
                 def func(x, *params):
                     return SHO_double(x, params[0], 0.0, params[-1])
                 p0 = [vrot, vlsr]
-                nparam = 2
+                priors = ['vrot', 'vlsr']
             elif (not fit_vrot) and fit_vrad:
                 def func(x, *params):
-                    return SHO_double(x, params[0], 0.0, params[-1])
-                p0 = [vrot, vlsr]
-                nparam = 2
+                    return SHO_double(x, 0.0, params[0], params[-1])
+                p0 = [vrad, vlsr]
+                priors = ['vrad', 'vlsr']
             else:
                 def func(x, *params):
                     return SHO_double(x, 0.0, 0.0, *params)
                 p0 = [vlsr]
-                nparam = 1
+                priors = ['vlsr']
         else:
             if fit_vrot and fit_vrad:
                 def func(x, *params):
                     return SHO_double(x, *params, fix_vlsr)
                 p0 = [vrot, vrad]
-                nparam = 2
+                priors = ['vrot', 'vrad']
             elif fit_vrot and (not fit_vrad):
                 def func(x, *params):
                     return SHO_double(x, *params, 0.0, fix_vlsr)
                 p0 = [vrot]
-                nparam = 1
+                priors = ['vrot']
             elif (not fit_vrot) and fit_vrad:
                 def func(x, *params):
                     return SHO_double(x, 0.0, *params, fix_vlsr)
                 p0 = [vrad]
-                nparam = 1
+                priors = ['vrad']
             else:
                 raise ValueError("You must fit one parameter!")
 
         # Returns if no values are provided.
 
         if len(y) == 0:
-            popt = np.empty(nparam)
+            popt = np.empty(len(p0))
             cvar = popt.copy()
             return popt, cvar
 
+        # Send values to with curve_fit or emcee.
+
+        if MCMC:
+            return self._SHO_MCMC(x, y, dy, func, p0, priors, optimize_kwargs)
+        else:
+            return self._SHO_chi2(x, y, dy, func, p0, optimize_kwargs)
+
+    def _SHO_chi2(self, x, y, dy, func, p0, optimize_kwargs=None):
+        """Use scipy.optimize.curve_fit for the fitting."""
+        from scipy.optimize import curve_fit
+
         # Set up the optimization.
 
         kw = {} if optimize_kwargs is None else optimize_kwargs
@@ -562,10 +601,73 @@ class rotationmap(datacube):
             popt, cvar = curve_fit(func, x, y, **kw)
             cvar = np.diag(cvar)**0.5
         except (ValueError, TypeError):
-            popt = np.empty(nparam)
+            popt = np.empty(len(p0))
             cvar = popt.copy()
         return popt, cvar
 
+    def _SHO_MCMC(self, x, y, dy, func, p0, priors, optimize_kwargs=None):
+        """Use an MCMC sampler to model the posteriors."""
+
+        # Set up the optimization.
+
+        kw = {} if optimize_kwargs is None else optimize_kwargs
+        if kw.pop('mcmc', 'emcee') == 'zeus':
+            EnsembleSampler = zeus.EnsembleSampler
+        else:
+            EnsembleSampler = emcee.EnsembleSampler
+        nwalkers = kw.pop('nwalkers', 128)
+        nburnin = kw.pop('nburnin', 200)
+        nsteps = kw.pop('nsteps', 100)
+        scatter = kw.pop('scatter', 1e-3)
+        progress = kw.pop('progress', True)
+        moves = kw.pop('moves', None)
+        pool = kw.pop('pool', None)
+
+        # Check that the starting positions are OK. If not, return NaN.
+        # TODO: Check how to handle this in the model making.
+
+        if not np.isfinite(np.sum([rotationmap.SHO_priors[p](t)
+                                   for t, p in zip(p0, priors)])):
+            return [np.nan for _ in p0], [np.nan for _ in p0]
+
+        # Run the EnsembleSampler
+
+        p0 = random_p0(p0, scatter, nwalkers)
+        sampler = EnsembleSampler(nwalkers,
+                                  p0.shape[1],
+                                  self._SHO_ln_probability,
+                                  args=[func, priors, x, y, dy],
+                                  moves=moves,
+                                  pool=pool)
+        sampler.run_mcmc(p0, nburnin + nsteps, progress=progress, **kw)
+
+        # Extract the median and (symmetric) uncertainties.
+
+        samples = sampler.get_chain(discard=nburnin, flat=True)
+        samples = np.percentile(samples, [16, 50, 84], axis=0)
+        return samples[1], 0.5 * (samples[2] - samples[0])
+
+    def _SHO_ln_probability(self, theta, func, priors, x, y, dy):
+        """Log-probablility function."""
+        lnp = self._SHO_ln_prior(theta, priors)
+        if np.isfinite(lnp):
+            return lnp + self._SHO_ln_likelihood(theta, func, x, y, dy)
+        return -np.inf
+
+    def _SHO_ln_prior(self, theta, priors):
+        """Priors for the MCMC fitting of the annuli."""
+        lnp = 0.0
+        for t, p in zip(theta, priors):
+            lnp += rotationmap.SHO_priors[p](t)
+            if not np.isfinite(lnp):
+                return lnp
+        return lnp
+
+    def _SHO_ln_likelihood(self, theta, func, x, y, dy):
+        """Log-likelihood for the MCMC fitting of the annuli."""
+        lnx2 = -0.5 * np.sum(np.power((y - func(x, *theta)) / dy, 2))
+        return lnx2 if np.isfinite(lnx2) else -np.inf
+
     def _get_radial_bins(self, rpnts=None, rbins=None):
         """Return default radial bins."""
         if rpnts is None and rbins is None:
@@ -599,12 +701,38 @@ class rotationmap(datacube):
             def prior(p):
                 if not min(args) <= p <= max(args):
                     return -np.inf
-                return np.log(1.0 / (args[1] - args[0]))
+                return max(-100.0, np.log(1.0 / (max(args) - min(args))))
         else:
             def prior(p):
                 return -0.5 * ((args[0] - p) / args[1])**2
         rotationmap.priors[param] = prior
 
+    def set_SHO_prior(self, param, args, type='flat'):
+        """
+        Set the prior for the given parameter for the SHO fitting. There are
+        two types of priors currently usable, ``'flat'`` which requires
+        ``args=[min, max]`` while for ``'gaussian'`` you need to specify
+        ``args=[mu, sig]``. The three ``params`` which are available are
+        ``'vrot'``, ``'vrad'`` and ``'vlsr'``.
+
+        Args:
+            param (str): Name of the parameter.
+            args (list): Values to use depending on the type of prior.
+            type (optional[str]): Type of prior to use.
+        """
+        type = type.lower()
+        if type not in ['flat', 'gaussian']:
+            raise ValueError("type must be 'flat' or 'gaussian'.")
+        if type == 'flat':
+            def prior(p):
+                if not min(args) <= p <= max(args):
+                    return -np.inf
+                return np.log(1.0 / (args[1] - args[0]))
+        else:
+            def prior(p):
+                return -0.5 * ((args[0] - p) / args[1])**2
+        rotationmap.SHO_priors[param] = prior
+
     def plot_data(self, vmin=None, vmax=None, ivar=None, return_fig=False):
         """
         Plot the first moment map. By default will clip the velocity contours
@@ -713,50 +841,58 @@ class rotationmap(datacube):
             return lnp + self._ln_likelihood(model)
         return -np.inf
 
+    def _load_default_parameters(self, path='default_parameters.yml'):
+        """Load the default parameters."""
+        with open(__file__.replace('rotationmap.py', path)) as stream:
+            parameters = yaml.safe_load(stream)
+        for p in parameters.keys():
+            if parameters[p]['prior_type'] is not None:
+                values = parameters[p]['prior_values']
+                if len(values) == 1:
+                    parameters[p]['prior_values'] = [-values[0], values[0]]
+        return parameters
+
+    def print_default_prior(self, parameter):
+        """Print the default prior for a given parameter."""
+        try:
+            prior_type = self.default_parameters[parameter]['prior_type']
+            prior_values = self.default_parameters[parameter]['prior_values']
+            str = '`{}` has a '
+            if prior_type == 'flat':
+                str += '{} prior with a minimum value of {}'
+                str += ' and a maximum value {}.'
+            elif prior_type == 'gaussian':
+                str += '{} prior with mean of {}'
+                str += ' and standard deviation of {}.'
+            print(str.format(parameter, prior_type, *prior_values))
+        except KeyError:
+            print('`{}` is not a free parameter.'.format(parameter))
+
     def _set_default_priors(self):
         """Set the default priors."""
 
-        # Basic Geometry.
+        # fit_map functions
 
-        self.set_prior('x0', [-0.5, 0.5], 'flat')
-        self.set_prior('y0', [-0.5, 0.5], 'flat')
-        self.set_prior('inc', [-90.0, 90.0], 'flat')
-        self.set_prior('PA', [-360.0, 360.0], 'flat')
-        self.set_prior('mstar', [0.1, 5.0], 'flat')
-        self.set_prior('vlsr', [np.nanmin(self.data),
-                                np.nanmax(self.data)], 'flat')
+        for k in self.default_parameters.keys():
+            p = self.default_parameters[k]
+            if p['prior_type'] is not None:
+                self.set_prior(k, p['prior_values'], p['prior_type'])
 
-        # Emission Surface
+        # SHO functions.
 
-        self.set_prior('z0', [0.0, 5.0], 'flat')
-        self.set_prior('psi', [0.0, 5.0], 'flat')
-        self.set_prior('r_cavity', [0.0, 1e30], 'flat')
-        self.set_prior('r_taper', [0.0, 1e30], 'flat')
-        self.set_prior('q_taper', [0.0, 15.0], 'flat')
-
-        # Warp
-
-        self.set_prior('w_i', [-90.0, 90.0], 'flat')
-        self.set_prior('w_r', [0.0, self.xaxis.max()], 'flat')
-        self.set_prior('w_t', [-180.0, 180.0], 'flat')
-
-        # Velocity Profile
-
-        self.set_prior('vp_100', [0.0, 1e4], 'flat')
-        self.set_prior('vp_q', [-2.0, 0.0], 'flat')
-        self.set_prior('vp_rtaper', [0.0, 1e30], 'flat')
-        self.set_prior('vp_qtaper', [0.0, 5.0], 'flat')
-        self.set_prior('vr_100', [-1e3, 1e3], 'flat')
-        self.set_prior('vr_q', [-2.0, 2.0], 'flat')
-        self.set_prior('r_pressure', [0.0, 3.0*self.xaxis.max()], 'flat')
-        self.set_prior('w_pressure', [0.0, 1e2], 'flat')
+        self.set_SHO_prior('vrot', [-3e3, 3e3], 'flat')
+        self.set_SHO_prior('vrad', [-1e2, 1e2], 'flat')
+        self.set_SHO_prior('vlsr', [-1e4, 1e4], 'flat')
 
     def _ln_prior(self, params):
         """Log-priors."""
         lnp = 0.0
         for key in params.keys():
             if key in rotationmap.priors.keys() and params[key] is not None:
-                lnp += rotationmap.priors[key](params[key])
+                try:
+                    lnp += rotationmap.priors[key](params[key])
+                except:
+                    print(key, params[key])
                 if not np.isfinite(lnp):
                     return lnp
         return lnp
@@ -793,7 +929,8 @@ class rotationmap(datacube):
 
         # Velocity mask.
 
-        v_min, v_max = params['v_min'], params['v_max']
+        v_min = params.get('v_min', np.nanmin(self.data))
+        v_max = params.get('v_max', np.nanmax(self.data))
         mask_v = np.logical_and(self.data >= v_min, self.data <= v_max)
         mask_v = ~mask_v if params['exclude_v'] else mask_v
 
@@ -837,96 +974,48 @@ class rotationmap(datacube):
         fitting. For non-essential parameters a default value is set.
         """
 
-        # Basic geometrical properties.
-
-        params['x0'] = params.pop('x0', 0.0)
-        params['y0'] = params.pop('y0', 0.0)
-        params['dist'] = params.pop('dist', 100.0)
-        params['vlsr'] = params.pop('vlsr', self.vlsr)
-        if params.get('inc', None) is None:
-            raise KeyError("Must provide `'inc'`.")
-        if params.get('PA', None) is None:
-            raise KeyError("Must provide `'PA'`.")
+        for p in self.default_parameters.keys():
+            params[p] = params.pop(p, self.default_parameters[p]['default'])
 
         # Rotation profile.
 
-        if params.get('r_pressure') is None:
-            params['has_pressure'] = False
-        else:
-            params['has_pressure'] = True
-
-        if params.get('vp_100') is None:
-            params['has_vp_100'] = False
-        else:
-            params['has_vp_100'] = True
-
-        if params.get('mstar') is None:
-            params['has_mstar'] = False
-        else:
-            params['has_mstar'] = True
-
-        if params['has_mstar'] and params['has_vp_100']:
-            raise KeyError("Only provide either `'mstar'` or `'vp_100'`.")
-        if not params['has_mstar'] and not params['has_vp_100']:
-            raise KeyError("Must provide either `'mstar'` or `'vp_100'`.")
-        if params['has_mstar']:
-            if params['has_pressure']:
-                params['vfunc'] = self._vkep_pressure
-            else:
-                params['vfunc'] = self._vkep
-        else:
-            if params['has_pressure']:
-                params['vfunc'] = self._vpow_pressure
-            else:
+        if params['vp_100'] is not None:
+            if params['mstar'] is not None:
                 params['vfunc'] = self._vpow
+            else:
+                raise ValueError("Cannot specify both `vp_100` and `mstar`.")
+        else:
+            params['vfunc'] = self._vkep
 
-        params['vp_q'] = params.pop('vp_q', -0.5)
-        params['vp_rtaper'] = params.pop('vp_rtaper', 1e10)
-        params['vp_qtaper'] = params.pop('vp_qtaper', 1.0)
-        params['vr_100'] = params.pop('vr_100', 0.0)
-        params['vr_q'] = params.pop('vr_q', 0.0)
-        params['w_pressure'] = params.pop('w_pressure', 0.0)
+        # Deprojection properties.
 
-        # Flared emission surface.
-
-        params['z0'] = params.pop('z0', None)
-        params['psi'] = params.pop('psi', None)
-        params['r_taper'] = params.pop('r_taper', None)
-        params['q_taper'] = params.pop('q_taper', None)
-        params['r_cavity'] = params.pop('r_cavity', None)
         params['z_func'] = params.pop('z_func', None)
-
-        # Warp parameters.
-
-        params['w_i'] = params.pop('w_i', 0.0)
-        params['w_r'] = params.pop('w_r', 0.5 * max(self.xaxis))
-        params['w_t'] = params.pop('w_t', 0.0)
         params['shadowed'] = params.pop('shadowed', False)
 
         # Masking parameters.
 
-        params['r_min'] = params.pop('r_min', 0.0)
-        params['r_max'] = params.pop('r_max', 1e10)
-        params['exclude_r'] = params.pop('exclude_r', False)
-        params['phi_min'] = params.pop('phi_min', -180.0)
-        params['phi_max'] = params.pop('phi_max', 180.0)
-        params['exclude_phi'] = params.pop('exclude_phi', False)
-        params['abs_phi'] = params.pop('abs_phi', False)
-        params['v_min'] = params.pop('v_min', np.nanmin(self.data))
-        params['v_max'] = params.pop('v_max', np.nanmax(self.data))
-        params['exclude_v'] = params.pop('exclude_v', False)
+        # params['r_min'] = params.pop('r_min', 0.0)
+        # params['r_max'] = params.pop('r_max', 1e10)
+        # params['exclude_r'] = params.pop('exclude_r', False)
+        # params['phi_min'] = params.pop('phi_min', -180.0)
+        # params['phi_max'] = params.pop('phi_max', 180.0)
+        # params['exclude_phi'] = params.pop('exclude_phi', False)
+        # params['abs_phi'] = params.pop('abs_phi', False)
+        # params['v_min'] = params.pop('v_min', np.nanmin(self.data))
+        # params['v_max'] = params.pop('v_max', np.nanmax(self.data))
+        # params['exclude_v'] = params.pop('exclude_v', False)
         params['user_mask'] = params.pop('user_mask', np.ones(self.data.shape))
 
-        if params['r_min'] >= params['r_max']:
-            raise ValueError("`r_max` must be greater than `r_min`.")
-        if params['phi_min'] >= params['phi_max']:
-            raise ValueError("`phi_max` must be great than `phi_min`.")
-        if params['v_min'] >= params['v_max']:
-            raise ValueError("`v_max` must be greater than `v_min`.")
+        # if params['r_min'] >= params['r_max']:
+        #     raise ValueError("`r_max` must be greater than `r_min`.")
+        # if params['phi_min'] >= params['phi_max']:
+        #     raise ValueError("`phi_max` must be great than `phi_min`.")
+        # if params['v_min'] >= params['v_max']:
+        #     raise ValueError("`v_max` must be greater than `v_min`.")
 
         # Beam convolution.
 
-        params['beam'] = bool(params.pop('beam', False))
+        # params['beam'] = bool(params.pop('beam', False))
 
         return params
 
@@ -1023,7 +1112,8 @@ class rotationmap(datacube):
     def save_model(self, samples=None, params=None, model=None, filename=None,
                    overwrite=True):
         """
-        Save the model as a FITS file.
+        Save the model as a FITS file. If you have used ``downsample`` when
+        loading the cube data, _this will not work_.
         """
         from astropy.io import fits
         if model is None:
@@ -1127,9 +1217,19 @@ class rotationmap(datacube):
         """Keplerian rotation velocity."""
         r_m = rvals * sc.au * params['dist']
         z_m = zvals * sc.au * params['dist']
-        vkep = sc.G * params['mstar'] * self.msun * np.power(r_m, 2)
+        mtotal = params['mstar'] + self._calc_mdisk(rvals, params)
+        vkep = sc.G * mtotal * self.msun * np.power(r_m, 2)
         return np.sqrt(vkep * np.power(np.hypot(r_m, z_m), -3))
 
+    def _calc_mdisk(self, rvals, params):
+        """Psuedo disk self-gravity component."""
+        if params['mdisk'] is None:
+            return 0.0
+        exponent = 2.0 - params['gamma']
+        rscale = rvals**exponent - params['r_in']**exponent
+        rscale /= params['r_out']**exponent - params['r_in']**exponent
+        return params['mdisk'] * np.clip(rscale, a_min=0.0, a_max=1.0)
+
     def _vkep_pressure(self, rvals, tvals, zvals, params):
         """Keplerian rotation velocity with pressure term."""
         vkep = self._vkep(rvals, tvals, zvals, params)
@@ -1463,6 +1563,7 @@ class rotationmap(datacube):
                 print("Change this at any time with `rotationmap.error`.")
                 self.error = 0.1 * (self.data - np.nanmedian(self.data))
         self.error = np.where(np.isnan(self.error), 0.0, abs(self.error))
+        self.error *= 1e3 if self.velocity_unit == 'km/s' else 1.0
         assert self.data.shape == self.error.shape
 
     # -- PLOTTING -- #
